@@ -17,7 +17,7 @@ from typing import Any, Dict, List, Match, Optional, Sequence, Type, Tuple
 from . import pq
 from . import errors as e
 from .pq import ExecStatus
-from .abc import ConnectionType, PQGen, Transformer
+from .abc import Buffer, ConnectionType, PQGen, Transformer
 from .adapt import PyFormat
 from ._compat import create_task
 from ._cmodule import _psycopg
@@ -51,9 +51,23 @@ class BaseCopy(Generic[ConnectionType]):
     """
 
     # Max size of the write queue of buffers. More than that copy will block
-    # Each buffer around Formatter.BUFFER_SIZE size
+    # Each buffer should be around BUFFER_SIZE size.
     QUEUE_SIZE = 1024
 
+    # Size of data to accumulate before sending it down the network. We fill a
+    # buffer this size field by field, and when it passes the threshold size
+    # wee ship it, so it may end up being bigger than this.
+    BUFFER_SIZE = 32 * 1024
+
+    # Maximum data size we want to queue to send to the libpq copy. Sending a
+    # buffer too big to be handled can cause an infinite loop in the libpq
+    # (#255) so we want to split it in more digestable chunks.
+    MAX_BUFFER_SIZE = 4 * BUFFER_SIZE
+    # Note: making this buffer too large, e.g.
+    # MAX_BUFFER_SIZE = 1024 * 1024
+    # makes operations *way* slower! Probably triggering some quadraticity
+    # in the libpq memory management and data sending.
+
     formatter: "Formatter"
 
     def __init__(self, cursor: "BaseCursor[ConnectionType, Any]"):
@@ -182,6 +196,7 @@ class Copy(BaseCopy["Connection[Any]"]):
         super().__init__(cursor)
         self._queue: queue.Queue[bytes] = queue.Queue(maxsize=self.QUEUE_SIZE)
         self._worker: Optional[threading.Thread] = None
+        self._worker_error: Optional[BaseException] = None
 
     def __enter__(self) -> "Copy":
         self._enter()
@@ -237,7 +252,7 @@ class Copy(BaseCopy["Connection[Any]"]):
         """
         return self.connection.wait(self._read_row_gen())
 
-    def write(self, buffer: Union[str, bytes]) -> None:
+    def write(self, buffer: Union[Buffer, str]) -> None:
         """
         Write a block of data to a table after a :sql:`COPY FROM` operation.
 
@@ -270,17 +285,22 @@ class Copy(BaseCopy["Connection[Any]"]):
     def worker(self) -> None:
         """Push data to the server when available from the copy queue.
 
-        Terminate reading when the queue receives a None.
+        Terminate reading when the queue receives a false-y value, or in case
+        of error.
 
         The function is designed to be run in a separate thread.
         """
-        while True:
-            data = self._queue.get(block=True, timeout=24 * 60 * 60)
-            if not data:
-                break
-            self.connection.wait(copy_to(self._pgconn, data))
+        try:
+            while True:
+                data = self._queue.get(block=True, timeout=24 * 60 * 60)
+                if not data:
+                    break
+                self.connection.wait(copy_to(self._pgconn, data))
+        except BaseException as ex:
+            # Propagate the error to the main thread.
+            self._worker_error = ex
 
-    def _write(self, data: bytes) -> None:
+    def _write(self, data: Buffer) -> None:
         if not data:
             return
 
@@ -290,7 +310,19 @@ class Copy(BaseCopy["Connection[Any]"]):
             self._worker.daemon = True
             self._worker.start()
 
-        self._queue.put(data)
+        # If the worker thread raies an exception, re-raise it to the caller.
+        if self._worker_error:
+            raise self._worker_error
+
+        if len(data) <= self.MAX_BUFFER_SIZE:
+            # Most used path: we don't need to split the buffer in smaller
+            # bits, so don't make a copy.
+            self._queue.put(data)
+        else:
+            # Copy a buffer too large in chunks to avoid causing a memory
+            # error in the libpq, which may cause an infinite loop (#255).
+            for i in range(0, len(data), self.MAX_BUFFER_SIZE):
+                self._queue.put(data[i : i + self.MAX_BUFFER_SIZE])
 
     def _write_end(self) -> None:
         data = self.formatter.end()
@@ -301,6 +333,10 @@ class Copy(BaseCopy["Connection[Any]"]):
             self._worker.join()
             self._worker = None  # break the loop
 
+        # Check if the worker thread raised any exception before terminating.
+        if self._worker_error:
+            raise self._worker_error
+
 
 class AsyncCopy(BaseCopy["AsyncConnection[Any]"]):
     """Manage an asynchronous :sql:`COPY` operation."""
@@ -344,7 +380,7 @@ class AsyncCopy(BaseCopy["AsyncConnection[Any]"]):
     async def read_row(self) -> Optional[Tuple[Any, ...]]:
         return await self.connection.wait(self._read_row_gen())
 
-    async def write(self, buffer: Union[str, bytes]) -> None:
+    async def write(self, buffer: Union[Buffer, str]) -> None:
         data = self.formatter.write(buffer)
         await self._write(data)
 
@@ -364,7 +400,7 @@ class AsyncCopy(BaseCopy["AsyncConnection[Any]"]):
     async def worker(self) -> None:
         """Push data to the server when available from the copy queue.
 
-        Terminate reading when the queue receives a None.
+        Terminate reading when the queue receives a false-y value.
 
         The function is designed to be run in a separate thread.
         """
@@ -374,14 +410,22 @@ class AsyncCopy(BaseCopy["AsyncConnection[Any]"]):
                 break
             await self.connection.wait(copy_to(self._pgconn, data))
 
-    async def _write(self, data: bytes) -> None:
+    async def _write(self, data: Buffer) -> None:
         if not data:
             return
 
         if not self._worker:
             self._worker = create_task(self.worker())
 
-        await self._queue.put(data)
+        if len(data) <= self.MAX_BUFFER_SIZE:
+            # Most used path: we don't need to split the buffer in smaller
+            # bits, so don't make a copy.
+            await self._queue.put(data)
+        else:
+            # Copy a buffer too large in chunks to avoid causing a memory
+            # error in the libpq, which may cause an infinite loop (#255).
+            for i in range(0, len(data), self.MAX_BUFFER_SIZE):
+                await self._queue.put(data[i : i + self.MAX_BUFFER_SIZE])
 
     async def _write_end(self) -> None:
         data = self.formatter.end()
@@ -399,9 +443,7 @@ class Formatter(ABC):
     """
 
     format: pq.Format
-
-    # Size of data to accumulate before sending it down the network
-    BUFFER_SIZE = 32 * 1024
+    BUFFER_SIZE = BaseCopy.BUFFER_SIZE
 
     def __init__(self, transformer: Transformer):
         self.transformer = transformer
@@ -413,7 +455,7 @@ class Formatter(ABC):
         ...
 
     @abstractmethod
-    def write(self, buffer: Union[str, bytes]) -> bytes:
+    def write(self, buffer: Union[Buffer, str]) -> bytes:
         ...
 
     @abstractmethod
@@ -439,7 +481,7 @@ class TextFormatter(Formatter):
         else:
             return None
 
-    def write(self, buffer: Union[str, bytes]) -> bytes:
+    def write(self, buffer: Union[Buffer, str]) -> Buffer:
         data = self._ensure_bytes(buffer)
         self._signature_sent = True
         return data
@@ -460,15 +502,14 @@ class TextFormatter(Formatter):
         buffer, self._write_buffer = self._write_buffer, bytearray()
         return buffer
 
-    def _ensure_bytes(self, data: Union[bytes, str]) -> bytes:
-        if isinstance(data, bytes):
-            return data
-
-        elif isinstance(data, str):
+    def _ensure_bytes(self, data: Union[Buffer, str]) -> Buffer:
+        if isinstance(data, str):
             return data.encode(self._encoding)
-
         else:
-            raise TypeError(f"can't write {type(data).__name__}")
+            # Assume, for simplicity, that the user is not passing stupid
+            # things to the write function. If that's the case, things
+            # will fail downstream.
+            return data
 
 
 class BinaryFormatter(Formatter):
@@ -493,7 +534,7 @@ class BinaryFormatter(Formatter):
 
         return parse_row_binary(data, self.transformer)
 
-    def write(self, buffer: Union[str, bytes]) -> bytes:
+    def write(self, buffer: Union[Buffer, str]) -> Buffer:
         data = self._ensure_bytes(buffer)
         self._signature_sent = True
         return data
@@ -533,15 +574,14 @@ class BinaryFormatter(Formatter):
         buffer, self._write_buffer = self._write_buffer, bytearray()
         return buffer
 
-    def _ensure_bytes(self, data: Union[bytes, str]) -> bytes:
-        if isinstance(data, bytes):
-            return data
-
-        elif isinstance(data, str):
+    def _ensure_bytes(self, data: Union[Buffer, str]) -> Buffer:
+        if isinstance(data, str):
             raise TypeError("cannot copy str data in binary mode: use bytes instead")
-
         else:
-            raise TypeError(f"can't write {type(data).__name__}")
+            # Assume, for simplicity, that the user is not passing stupid
+            # things to the write function. If that's the case, things
+            # will fail downstream.
+            return data
 
 
 def _format_row_text(
